@@ -1,20 +1,24 @@
 use super::{ClarityInterpreter, ExecutionResult};
 use crate::clarity::analysis::ContractAnalysis;
 use crate::clarity::ast::ContractAST;
+use crate::clarity::codec::StacksMessageCodec;
 use crate::clarity::coverage::{CoverageReporter, TestCoverageReport};
 use crate::clarity::docs::{make_api_reference, make_define_reference, make_keyword_reference};
+use crate::clarity::errors::Error;
 use crate::clarity::functions::define::DefineFunctions;
 use crate::clarity::functions::NativeFunctions;
-use crate::clarity::types::{PrincipalData, QualifiedContractIdentifier, StandardPrincipalData};
+use crate::clarity::types::{
+    PrincipalData, QualifiedContractIdentifier, StandardPrincipalData, Value,
+};
 use crate::clarity::util::StacksAddress;
 use crate::clarity::variables::NativeVariables;
-use crate::clarity::errors::Error;
 use crate::contracts::{BNS_CONTRACT, COSTS_V1_CONTRACT, COSTS_V2_CONTRACT, POX_CONTRACT};
 use crate::repl::CostSynthesis;
 use crate::{clarity::diagnostic::Diagnostic, repl::settings::InitialContract};
 use ansi_term::{Colour, Style};
-use serde_json::Value;
 use std::collections::{BTreeMap, BTreeSet, HashMap, VecDeque};
+use std::fmt;
+use std::num::ParseIntError;
 use std::sync::{Arc, Mutex};
 
 #[cfg(feature = "cli")]
@@ -47,6 +51,7 @@ pub struct CostsReport {
 pub struct Session {
     session_id: u32,
     started_at: u32,
+    pub is_interactive: bool,
     pub settings: SessionSettings,
     pub contracts: BTreeMap<String, BTreeMap<String, Vec<String>>>,
     pub asts: BTreeMap<QualifiedContractIdentifier, ContractAST>,
@@ -72,7 +77,12 @@ impl Session {
         Session {
             session_id: 0,
             started_at: 0,
-            interpreter: ClarityInterpreter::new(tx_sender, settings.costs_version),
+            is_interactive: false,
+            interpreter: ClarityInterpreter::new(
+                tx_sender,
+                settings.costs_version,
+                settings.analysis.clone(),
+            ),
             asts: BTreeMap::new(),
             contracts: BTreeMap::new(),
             api_reference: build_api_reference(),
@@ -87,7 +97,7 @@ impl Session {
     async fn retrieve_contract(
         &mut self,
         link: &InitialLink,
-    ) -> Result<(String, BTreeSet<QualifiedContractIdentifier>), String> {
+    ) -> Result<(String, Vec<QualifiedContractIdentifier>), String> {
         let contract_id = &link.contract_id;
         let components: Vec<&str> = contract_id.split('.').collect();
         let contract_deployer = components.first().expect("");
@@ -139,7 +149,7 @@ impl Session {
 
             // Extract principal from contract_id
             let (contract_code, deps) = match handled.get(contract_id) {
-                Some(entry) => (entry.clone(), BTreeSet::new()),
+                Some(entry) => (entry.clone(), Vec::new()),
                 None => {
                     let (contract_code, deps) = self
                         .retrieve_contract(&initial_link)
@@ -178,6 +188,7 @@ impl Session {
     #[cfg(not(feature = "wasm"))]
     pub fn start(&mut self) -> Result<(String, Vec<(ContractAnalysis, String, String)>), String> {
         let mut output_err = Vec::<String>::new();
+        let mut output = vec![];
         let mut contracts = vec![];
 
         if !self.settings.include_boot_contracts.is_empty() {
@@ -234,7 +245,7 @@ impl Session {
                     true,
                     None,
                 ) {
-                    Ok(_) => {}
+                    Ok((mut output, _)) => output_err.append(&mut output),
                     Err(ref mut result) => output_err.append(result),
                 };
             }
@@ -283,7 +294,8 @@ impl Session {
                     true,
                     Some("Deployment".into()),
                 ) {
-                    Ok((_, result)) => {
+                    Ok((ref mut res_output, result)) => {
+                        output.append(res_output);
                         if result.contract.is_none() {
                             continue;
                         }
@@ -317,15 +329,18 @@ impl Session {
             });
         }
 
-        let mut output = vec![];
-        if !self.settings.initial_contracts.is_empty() {
-            output.push(blue!("Contracts"));
-            self.get_contracts(&mut output);
-        }
+        if self.is_interactive {
+            // If the session is interactive (clarinet console, usr/bin/clarity-repl)
+            // we will display the contracts + genesis asset map.
+            if !self.settings.initial_contracts.is_empty() {
+                output.push(blue!("Contracts"));
+                self.get_contracts(&mut output);
+            }
 
-        if self.settings.initial_accounts.len() > 0 {
-            output.push(blue!("Initialized balances"));
-            self.get_accounts(&mut output);
+            if self.settings.initial_accounts.len() > 0 {
+                output.push(blue!("Initialized balances"));
+                self.get_accounts(&mut output);
+            }
         }
 
         self.initial_contracts_analysis
@@ -449,7 +464,7 @@ impl Session {
                     true,
                     None,
                 ) {
-                    Ok(_) => {}
+                    Ok((ref mut res_output, _)) => output.append(res_output),
                     Err(ref mut result) => output.append(result),
                 };
             }
@@ -501,6 +516,8 @@ impl Session {
                 self.parse_and_advance_chain_tip(&mut output, cmd)
             }
             cmd if cmd.starts_with("::toggle_costs") => self.toggle_costs(&mut output),
+            cmd if cmd.starts_with("::encode") => self.encode(&mut output, cmd),
+            cmd if cmd.starts_with("::decode") => self.decode(&mut output, cmd),
 
             snippet => {
                 if self.show_costs {
@@ -512,7 +529,13 @@ impl Session {
                         true,
                         None,
                     ) {
-                        Ok((result, _)) => result,
+                        Ok((mut output, result)) => {
+                            if let Some((ref contract_name, _, _, _, _)) = result.contract {
+                                let snippet = format!("→ .{} contract successfully stored. Use (contract-call? ...) for invoking the public functions:", contract_name.clone());
+                                output.push(green!(snippet));
+                            }
+                            output
+                        }
                         Err(result) => result,
                     };
                     output.append(&mut result);
@@ -533,12 +556,14 @@ impl Session {
 
         let result = self.interpret(snippet.to_string(), name.clone(), cost_track, test_name);
         let mut output = Vec::<String>::new();
+        let lines = snippet.lines();
+        let formatted_lines: Vec<String> = lines.map(|l| l.to_string()).collect();
+        let contract_name = name.unwrap_or("<stdin>".to_string());
 
         match result {
             Ok(result) => {
-                if let Some((ref contract_name, _, _, _, _)) = result.contract {
-                    let snippet = format!("→ .{} contract successfully stored. Use (contract-call? ...) for invoking the public functions:", contract_name.clone());
-                    output.push(green!(snippet));
+                for diagnostic in &result.diagnostics {
+                    output.append(&mut diagnostic.output(&contract_name, &formatted_lines));
                 }
                 if result.events.len() > 0 {
                     output.push(black!("Events emitted"));
@@ -547,63 +572,13 @@ impl Session {
                     }
                 }
                 if let Some(ref result) = result.result {
-                    output.push(green!(result));
+                    output.push(green!(format!("{}", result)));
                 }
                 Ok((output, result))
             }
-            Err((message, diagnostic, _)) => {
-                if let Some(name) = name {
-                    output.push(format!("Error found in contract {}", name)); 
-                }
-                output.push(red!(message));
+            Err((_, diagnostic, _)) => {
                 if let Some(diagnostic) = diagnostic {
-                    if diagnostic.spans.len() > 0 {
-                        let lines = snippet.lines();
-                        let mut formatted_lines: Vec<String> =
-                            lines.map(|l| l.to_string()).collect();
-                        for span in diagnostic.spans {
-                            let first_line = span.start_line.saturating_sub(1) as usize;
-                            let last_line = span.end_line.saturating_sub(1) as usize;
-                            let mut pass = vec![];
-
-                            for (line_index, line) in formatted_lines.iter().enumerate() {
-                                if line == "" {
-                                    pass.push(line.clone());
-                                    continue;
-                                }
-                                if line_index >= first_line && line_index <= last_line {
-                                    let (begin, end) =
-                                        match (line_index == first_line, line_index == last_line) {
-                                            (true, true) => (
-                                                span.start_column.saturating_sub(1) as usize,
-                                                span.end_column.saturating_sub(1) as usize,
-                                            ), // One line
-                                            (true, false) => (
-                                                span.start_column.saturating_sub(1) as usize,
-                                                line.len().saturating_sub(1),
-                                            ), // Multiline, first line
-                                            (false, false) => (0, line.len().saturating_sub(1)), // Multiline, in between
-                                            (false, true) => {
-                                                (0, span.end_column.saturating_sub(1) as usize)
-                                            } // Multiline, last line
-                                        };
-
-                                    let error_style = light_red.underline();
-                                    let formatted_line = format!(
-                                        "{}{}{}",
-                                        &line[..begin],
-                                        error_style.paint(&line[begin..=end]),
-                                        &line[(end + 1)..]
-                                    );
-                                    pass.push(formatted_line);
-                                } else {
-                                    pass.push(line.clone());
-                                }
-                            }
-                            formatted_lines = pass;
-                        }
-                        output.append(&mut formatted_lines);
-                    }
+                    output.append(&mut diagnostic.output(&contract_name, &formatted_lines));
                 }
                 Err(output)
             }
@@ -852,6 +827,55 @@ impl Session {
         output.push(green!(format!("Always show costs: {}", self.show_costs)))
     }
 
+    pub fn encode(&mut self, output: &mut Vec<String>, cmd: &str) {
+        let snippet = match cmd.split_once(" ") {
+            Some((_, snippet)) => snippet,
+            _ => return output.push(red!("Usage: ::encode <expr>")),
+        };
+
+        let result = self.interpret(snippet.to_string(), None, false, None);
+        let value = match result {
+            Ok(result) => {
+                let mut tx_bytes = vec![];
+                let value = match result.result {
+                    Some(value) => value,
+                    None => return output.push("No value".to_string()),
+                };
+                match value.consensus_serialize(&mut tx_bytes) {
+                    Err(e) => return output.push(red!(format!("{}", e))),
+                    _ => (),
+                };
+                let mut s = String::with_capacity(2 * tx_bytes.len());
+                for byte in tx_bytes {
+                    s = format!("{}{:02x}", s, byte);
+                }
+                green!(s)
+            }
+            Err((_, Some(diagnostic), Some(e))) => red!(format!("{}: {}", diagnostic.message, e)),
+            Err((_, Some(diagnostic), None)) => red!(format!("{}", diagnostic.message)),
+            Err((_, None, Some(e))) => red!(format!("{}", e)),
+            _ => panic!("Result of interpret has neither diagnostic or error"),
+        };
+        output.push(value);
+    }
+
+    pub fn decode(&mut self, output: &mut Vec<String>, cmd: &str) {
+        let byteString = match cmd.split_once(" ") {
+            Some((_, bytes)) => bytes,
+            _ => return output.push(red!("Usage: ::decode <hex-bytes>")),
+        };
+        let tx_bytes = match decode_hex(byteString) {
+            Ok(tx_bytes) => tx_bytes,
+            Err(e) => return output.push(red!(format!("Parsing error: {}", e))),
+        };
+
+        let value = match Value::consensus_deserialize(&mut &tx_bytes[..]) {
+            Ok(value) => value,
+            Err(e) => return output.push(red!(format!("{}", e))),
+        };
+        output.push(green!(format!("{}", value)));
+    }
+
     #[cfg(feature = "cli")]
     pub fn get_costs(&mut self, output: &mut Vec<String>, cmd: &str) {
         let snippet = cmd.to_string().split_off("::get_costs ".len());
@@ -1060,6 +1084,47 @@ impl Session {
     }
 }
 
+#[derive(Debug, PartialEq)]
+enum DecodeHexError {
+    ParseError(ParseIntError),
+    OddLength,
+}
+
+impl fmt::Display for DecodeHexError {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match self {
+            DecodeHexError::ParseError(e) => write!(f, "{}", e),
+            DecodeHexError::OddLength => write!(f, "odd number of hex digits"),
+        }
+    }
+}
+
+impl From<ParseIntError> for DecodeHexError {
+    fn from(err: ParseIntError) -> Self {
+        DecodeHexError::ParseError(err)
+    }
+}
+
+fn decode_hex(byteString: &str) -> Result<Vec<u8>, DecodeHexError> {
+    let byteStringFiltered: String = byteString
+        .strip_prefix("0x")
+        .unwrap_or(byteString)
+        .chars()
+        .filter(|c| !c.is_whitespace())
+        .collect();
+    if byteStringFiltered.len() % 2 != 0 {
+        return Err(DecodeHexError::OddLength);
+    }
+    let result: Result<Vec<u8>, ParseIntError> = (0..byteStringFiltered.len())
+        .step_by(2)
+        .map(|i| u8::from_str_radix(&byteStringFiltered[i..i + 2], 16))
+        .collect();
+    match result {
+        Ok(result) => Ok(result),
+        Err(e) => Err(DecodeHexError::ParseError(e)),
+    }
+}
+
 #[derive(Deserialize, Debug, Default, Clone)]
 struct Contract {
     source: String,
@@ -1118,3 +1183,85 @@ fn build_api_reference() -> HashMap<String, String> {
     }
     api_reference
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn encode_simple() {
+        let mut session = Session::new(SessionSettings::default());
+        let mut output: Vec<String> = Vec::new();
+        session.encode(&mut output, "::encode 42");
+        assert_eq!(output.len(), 1);
+        assert_eq!(output[0], green!("000000000000000000000000000000002a"));
+    }
+
+    #[test]
+    fn encode_map() {
+        let mut session = Session::new(SessionSettings::default());
+        let mut output: Vec<String> = Vec::new();
+        session.encode(&mut output, "::encode { foo: \"hello\", bar: false }");
+        assert_eq!(output.len(), 1);
+        assert_eq!(
+            output[0],
+            green!("0c00000002036261720403666f6f0d0000000568656c6c6f")
+        );
+    }
+
+    #[test]
+    fn encode_error() {
+        let mut session = Session::new(SessionSettings::default());
+        let mut output: Vec<String> = Vec::new();
+        session.encode(&mut output, "::encode { foo false }");
+        assert_eq!(output.len(), 1);
+        assert_eq!(
+            output[0],
+            red!("Tuple literal construction expects a colon at index 1")
+        );
+
+        session.encode(&mut output, "::encode (foo 1)");
+        assert_eq!(output.len(), 2);
+        assert_eq!(output[1], red!("use of unresolved function 'foo'"));
+    }
+
+    #[test]
+    fn decode_simple() {
+        let mut session = Session::new(SessionSettings::default());
+        let mut output: Vec<String> = Vec::new();
+        session.decode(&mut output, "::decode 0000000000000000 0000000000000000 2a");
+        assert_eq!(output.len(), 1);
+        assert_eq!(output[0], green!("42"));
+    }
+
+    #[test]
+    fn decode_map() {
+        let mut session = Session::new(SessionSettings::default());
+        let mut output: Vec<String> = Vec::new();
+        session.decode(
+            &mut output,
+            "::decode 0x0c00000002036261720403666f6f0d0000000568656c6c6f",
+        );
+        assert_eq!(output.len(), 1);
+        assert_eq!(output[0], green!("{bar: false, foo: \"hello\"}"));
+    }
+
+    #[test]
+    fn decode_error() {
+        let mut session = Session::new(SessionSettings::default());
+        let mut output: Vec<String> = Vec::new();
+        session.decode(&mut output, "::decode 42");
+        assert_eq!(output.len(), 1);
+        assert_eq!(
+            output[0],
+            red!("Failed to decode clarity value: Deserialization error: Bad type prefix")
+        );
+
+        session.decode(&mut output, "::decode 4g");
+        assert_eq!(output.len(), 2);
+        assert_eq!(
+            output[1],
+            red!("Parsing error: invalid digit found in string")
+        );
+    }
+}
